/*
 *  Copyright 2010 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

Ext.namespace('org.ametys.explorer.view');


/**
 * org.ametys.explorer.view.ImageThumbnailViewer
 *
 * @class This class provides a data viewer for thumbnail images
 * @extends Ext.DataView
 * @constructor
 * @param {Object} config Configuration options
 */
org.ametys.explorer.view.ImageThumbnailViewer = function(config) 
{
	config.cls = 'images-view';
	config.autoHeight = true;
	config.multiSelect = true; 
	config.overClass = 'x-view-over';      
	config.itemSelector = 'div.thumb-wrap';
	
	if (!config.store)
		config.store = this._store;
    
	if (!config.tpl)
		config.tpl = this._tpl;
	
	this._labelEditor = new Ext.DataView.LabelEditor({dataIndex: 'name'});
	config.plugins = [
              //new Ext.DataView.DragSelector(),
              this._labelEditor
          ];
	
	config.listeners = {'render': {fn: this._initializeDragZone, scope: this}};
		
	org.ametys.explorer.view.ImageThumbnailViewer.superclass.constructor.call(this, config);
};

Ext.extend(org.ametys.explorer.view.ImageThumbnailViewer, Ext.DataView, {
	autoScroll: true
});

/**
 * @private
 * The default template. Can be override in configuration
 */
org.ametys.explorer.view.ImageThumbnailViewer.prototype._tpl = new Ext.XTemplate(
		'<tpl for=".">',
		'<div class="thumb-wrap" id="{name}">',
		'<table width="110" heigth="130" cellpadding="0" cellspacing="0">',
		'<tr><td valign="middle" class="thumb">',
		'<img src="{thumbnail}" title="{name}">',
		'</td></tr>',
		'<tr><td valign="middle">',
		'<span class="x-editable">{shortName}</span>',
		'</td></tr>',
		'</table>',
		'</div>',
		'</tpl>',
		'<div class="x-clear"></div>'
);

/**
 * @private
 * The default store. Can be override in configuration
 */
org.ametys.explorer.view.ImageThumbnailViewer.prototype._store = new Ext.data.ArrayStore (
		{
			fields: ['name', 
			         'url', 
			         {name:'size', type: 'float'}, 
			         {name:'lastModified', type:'date', dateFormat:'timestamp'}
			],
			idIndex: 0 // id for each record will be the first element
		}
);

org.ametys.explorer.view.ImageThumbnailViewer.prototype.startEditRecord = function (id)
{
	var index = this.getStore().find('id', id)
	var record = this.getStore().getAt(index);
	
	if (record != null)
	{
		var recordEl =  this.getNode(record);
		var target = recordEl.getElementsByClassName('x-editable')[0];
		
		this._labelEditor.startEdit(target, record.data['name']);
		this._labelEditor.activeRecord = record;
	}
}

org.ametys.explorer.view.ImageThumbnailViewer.prototype.prepareData = function (data)
{
	data.shortName = Ext.util.Format.ellipsis(data.name, 15);
	data.sizeString = Ext.util.Format.fileSize(data.size);
	data.dateString = data.lastModified.format("m/d/Y g:i a");
	
	var filename = data.name;
	var extension = "unknown";
	var index = filename.indexOf('.');
	if (index > 0)
	{
		extension = filename.substring(index + 1).toLowerCase();
	}
	
	var filename = data.name.toLowerCase();
	if (extension == 'jpg' || extension == 'jpeg' || extension == 'gif' || extension == 'png')
	{
		data.thumbnail = getPluginDirectUrl('explorer') + "/resource?id=" + data.id + "&maxWidth=100&maxHeight=100";;
	}
	else
	{
		data.thumbnail = getPluginDirectUrl('explorer') + "/thumbnail/" + extension + ".png";
	}
  	
	
	return data;
}

org.ametys.explorer.view.ImageThumbnailViewer.prototype._initializeDragZone = function (view) 
{
	view._dragTpl = new Ext.Template (
			'&lt;dl id="ext-gen317" class=" x-list-over"&gt;',
			'&lt;dt style="width: 40%; text-align: left;"&gt;',
			'&lt;em unselectable="on"&gt;',
			'&lt;img class="icon" src="{iconPath}"/&gt;{name}',
			'&lt;/em&gt;',
			'&lt;dt style="width: 20%; text-align: right;"&gt;',
			'&lt;em unselectable="on"&gt;{sizeString}&lt;/em&gt;',
			'&lt;/dt&gt;&lt;/dl&gt;'
	);
	
	view._dragTpl.compile();
	
	view._preventSelection = false;
	
	view.dragZone = new Ext.dd.DragZone(view.getEl(), {
		
		ddGroup: 'explorerGp',

		collectSelection: function(data) {
	        data.repairXY = Ext.fly(view.getSelectedNodes()[0]).getXY();
	        var i = 0;
	        view.store.each(function(rec){
	            if (view.isSelected(i)) {
	                var n = view.getNode(i);
	                var dragNode = n.cloneNode(true);
	                dragNode.id = Ext.id();
	                var ddel = view._dragTpl.overwrite(dragNode, view.store.getAt(i).data)
	                data.ddel.appendChild(ddel);
	                data.records.push(view.store.getAt(i));
	                data.viewNodes.push(n);
	            }
	            i++;
	        }, this);
	    },
	    
	    onBeforeDrag: function (data, e)
	    {
	    },
	    
		// On receipt of a mousedown event, see if it is within a draggable element.
		// Return a drag data object if so. The data object can contain arbitrary application
		// data, but it should also contain a DOM element in the ddel property to provide
		// a proxy to drag.
        getDragData: function(e) 
        {
            var sourceEl = e.getTarget(view.itemSelector, 10);
            if (sourceEl) 
            {
                /*d = sourceEl.cloneNode(true);
                d.id = Ext.id();
                data = view.getRecord(sourceEl).data;
                data.type = 'resource';
                return view.dragData = {
                    sourceEl: sourceEl,
                    repairXY: Ext.fly(sourceEl).getXY(),
                    ddel: view._dragTpl.overwrite(d, view.getRecord(sourceEl).data),
                    appData: data
                }
                */
            	if (view.getSelectionCount() == 0 || e.ctrlKey || e.shiftKey) return false;

                var dragData = {
                    sourceView: view,
                    viewNodes: [],
                    records: [],
                    recordType: 'resource'
                };
                
                if (view.getSelectionCount() == 1) 
                {
                	var i = view.getSelectedIndexes()[0];
                    var n = view.getNode(i);
                    dragData.viewNodes.push(dragData.ddel = n);
                    dragData.records.push(view.store.getAt(i));
                    dragData.repairXY = Ext.fly(n).getXY();
                }
                else 
                {
                    dragData.ddel = document.createElement('div');
                    dragData.ddel.className = 'multi-proxy';
                    this.collectSelection(dragData);
                }
                return dragData;
            }
        },

		// Provide coordinates for the proxy to slide back to on failed drag.
		// This is the original XY coordinates of the draggable element.
        getRepairXY: function() 
        {
            return this.dragData.repairXY;
        }
    });
}